/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Event.NativePreviewEvent;
import com.google.gwt.user.client.ui.RootPanel;
import com.sencha.gxt.core.client.Style.AnchorAlignment;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.BaseEventPreview;
import com.sencha.gxt.desktop.client.theme.base.startmenu.StartMenuBaseAppearance;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.event.BeforeHideEvent;
import com.sencha.gxt.widget.core.client.event.BeforeShowEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuItem;

public class StartMenu extends Component {

  public interface Appearance {

    public void setHeading(XElement parent, MenuItem heading);

    public void setMainMenu(XElement parent, Menu mainMenu);

    public void setToolMenu(XElement parent, Menu toolMenu);

    void render(SafeHtmlBuilder builder);

  }

  private StartHeadingMenuItem heading;
  private StartMainMenu mainMenu;
  private StartToolMenu toolMenu;
  private BaseEventPreview eventPreview;
  private boolean showing;

  public StartMenu() {
    this(GWT.<StartMenuBaseAppearance> create(StartMenuBaseAppearance.class));
  }

  public StartMenu(StartMenuBaseAppearance appearance) {

    SafeHtmlBuilder builder = new SafeHtmlBuilder();
    appearance.render(builder);

    XElement element = XDOM.create(builder.toSafeHtml());
    setElement(element);

    appearance.setHeading(element, getHeading());
    appearance.setMainMenu(element, getMainMenu());
    appearance.setToolMenu(element, getToolMenu());

    sinkEvents(Event.MOUSEEVENTS | Event.ONCLICK);
  }

  public void add(StartMainMenuItem menuItem) {
    getMainMenu().add(menuItem);
  }

  public void addTool(StartToolMenuItem startToolMenuItem) {
    getToolMenu().add(startToolMenuItem);
  }

  public void addToolSeparator() {
    // TODO: Provide implementation
  }

  /**
   * Hides the menu.
   */
  public void hide() {
    if (showing) {
      if (fireCancellableEvent(new BeforeHideEvent())) {
        showing = false;
        onHide();
        RootPanel.get().remove(this);
        eventPreview.remove();
        fireEvent(new HideEvent());
      }
    }
  }

  public void setHeading(String text) {
    getHeading().setText(text);
  }

  public void setIcon(ImageResource icon) {
    getHeading().setIcon(icon);
  }

  /**
   * Displays this menu relative to another element.
   * 
   * @param elem the element to align to
   * @param alignment the {@link XElement#alignTo} anchor position to use in
   *          aligning to the element (defaults to defaultAlign)
   * @param offsets the menu align offsets
   */
  public void show(Element elem, AnchorAlignment alignment, int[] offsets) {
    if (fireCancellableEvent(new BeforeShowEvent())) {
      showing = true;
      RootPanel.get().add(this);
      getElement().makePositionable(true);
      onShow();
      getElement().updateZIndex(0);
      doAutoSize();
      getElement().alignTo(elem, alignment, offsets);
      // TODO: Add support for scrolling as in Menu.show
      getElement().show();
      getEventPreview().add();
      focus();
      fireEvent(new ShowEvent());
    }
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    ComponentHelper.doAttach(heading);
    ComponentHelper.doAttach(mainMenu);
    ComponentHelper.doAttach(toolMenu);
  }

  protected void doAutoSize() {
    String w = getElement().getStyle().getWidth();
    if (showing && "".equals(w)) {
      // TODO: get main menu container from appearance as in Menu.doAutoSize
      getMainMenu().getElement().setWidth(195, true);
    }
  }

  @Override
  protected void doDetachChildren() {
    super.doDetachChildren();
    ComponentHelper.doDetach(heading);
    ComponentHelper.doDetach(mainMenu);
    ComponentHelper.doDetach(toolMenu);
  }

  private BaseEventPreview getEventPreview() {
    if (eventPreview == null) {
      eventPreview = new BaseEventPreview() {

        protected boolean onAutoHide(NativePreviewEvent pe) {
          // TODO: Add support for sub-menus as in Menu.onAutoHide
          hide();
          return true;
        }

        @Override
        protected void onPreviewKeyPress(NativePreviewEvent pe) {
          if (pe.getNativeEvent().getKeyCode() == KeyCodes.KEY_ESCAPE) {
            hide();
          }
        };

      };
      eventPreview.getIgnoreList().add(getMainMenu().getElement());
      eventPreview.getIgnoreList().add(getToolMenu().getElement());
    }
    return eventPreview;
  }

  private StartHeadingMenuItem getHeading() {
    if (heading == null) {
      heading = new StartHeadingMenuItem();
    }
    return heading;
  }

  private StartMainMenu getMainMenu() {
    if (mainMenu == null) {
      mainMenu = new StartMainMenu(this);
    }
    return mainMenu;
  }

  private StartToolMenu getToolMenu() {
    if (toolMenu == null) {
      toolMenu = new StartToolMenu(this);
    }
    return toolMenu;
  }
}
