/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.user.client.History;
import com.google.gwt.user.client.ui.RootPanel;
import com.sencha.gxt.desktop.client.desktopapp.event.LoginEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.LoginHandler;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateSettingsEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateSettingsHandler;
import com.sencha.gxt.desktop.client.desktopapp.utility.Prompt;
import com.sencha.gxt.desktop.client.service.login.LoginServiceProvider;
import com.sencha.gxt.desktop.client.service.settings.SettingsServiceProvider;

public class DesktopApp implements EntryPoint {

  private DesktopBus desktopBus;
  private Presenter presenter;
  private View view;
  private UpdateSettingsHandler updateSettingsHandler;
  private LoginHandler loginHandler;

  @Override
  public void onModuleLoad() {
    if (!getPresenter().isLocalStorageSupported()) {
      loadApplicationAfterAlertingUser();
    } else {
      loadApplication();
    }
  }

  private void checkForLogin() {
    if (!getPresenter().isLoggedIn()) {
      getDesktopBus().requestLoginService(getPresenter());
    } else {
      displayView();
    }
  }

  private void displayView() {
    getView().go(RootPanel.get());
  }

  private DesktopBus getDesktopBus() {
    if (desktopBus == null) {
      desktopBus = new DesktopBus();
      desktopBus.addLoginService(new LoginServiceProvider(desktopBus));
      desktopBus.addSettingsService(new SettingsServiceProvider(desktopBus));
      desktopBus.addUpdateSettingsHandler(getUpdateSettingsHandler());
      desktopBus.addLoginHandler(getLoginHandler());
    }
    return desktopBus;
  }

  private LoginHandler getLoginHandler() {
    if (loginHandler == null) {
      loginHandler = new LoginHandler() {
        @Override
        public void onLogin(LoginEvent event) {
          // If new user, request settings update service
          if (event.isNew()) {
            Settings settings = getPresenter().getSettings(event.getUserName());
            getDesktopBus().requestSettingsService(settings);
          } else {
            displayView();
          }
        }
      };
    }
    return loginHandler;
  }

  private Presenter getPresenter() {
    if (presenter == null) {
      presenter = new DesktopAppPresenter(getDesktopBus());
    }
    return presenter;
  }

  private UpdateSettingsHandler getUpdateSettingsHandler() {
    if (updateSettingsHandler == null) {
      updateSettingsHandler = new UpdateSettingsHandler() {
        @Override
        public void onUpdateSettings(UpdateSettingsEvent event) {
          // If view not yet displayed (i.e. first update for new user)
          if (view == null) {
            displayView();
          }
        }
      };
    }
    return updateSettingsHandler;
  }

  private View getView() {
    if (view == null) {
      view = new DesktopAppView(getDesktopBus(), getPresenter());
    }
    return view;
  }

  private void loadApplication() {
    if ("clear".equals(History.getToken())) {
      promptToClearStorage();
    } else {
      checkForLogin();
    }
  }

  private void loadApplicationAfterAlertingUser() {
    Prompt.get().alert(
        "Local Storage is Not Supported",
        "Either your browser does not support HTML5 Local Storage or it is not configured for use by this application.<br/><br/>This application will continue to run, but anything you create will be discarded when the browser terminates or the browser window is refreshed.",
        new Runnable() {
          @Override
          public void run() {
            loadApplication();
          }
        });
  }

  private void promptToClearStorage() {
    Prompt.get().confirm("Desktop", "Would you like to clear this application's local storage before beginning?",
        new Runnable() {
          @Override
          public void run() {
            getPresenter().clearLocalStorage();
            checkForLogin();
          }
        }, new Runnable() {
          @Override
          public void run() {
            checkForLogin();
          }
        });
  }

}
