/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp;

import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.ui.HasWidgets;
import com.sencha.gxt.desktop.client.Desktop;
import com.sencha.gxt.desktop.client.Shortcut;
import com.sencha.gxt.desktop.client.StartMainMenuItem;
import com.sencha.gxt.desktop.client.StartToolMenuItem;
import com.sencha.gxt.desktop.client.desktopapp.browser.BrowserView;
import com.sencha.gxt.desktop.client.desktopapp.event.OpenFileModelEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.OpenFileModelHandler;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateSettingsEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateSettingsHandler;
import com.sencha.gxt.desktop.client.desktopapp.filemanager.FileManagerView;
import com.sencha.gxt.desktop.client.desktopapp.has.AsWindow;
import com.sencha.gxt.desktop.client.desktopapp.images.DesktopImages;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileSystem;
import com.sencha.gxt.desktop.client.desktopapp.wordprocessor.WordProcessorView;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
import com.sencha.gxt.widget.core.client.info.Info;
import com.sencha.gxt.widget.core.client.menu.Item;

public class DesktopAppView implements View {

  // TODO: Consider replacing getData/setData with strongly typed properties
  private static final String DESKLET_KEY = "desklet";

  private Desktop desktop;
  private FileManagerView fileManagerView;
  private Shortcut fileManagerShortcut;
  private StartMainMenuItem fileManagerStartMenuItem;
  private StartToolMenuItem settingsToolMenuItem;
  private StartToolMenuItem logoutToolMenuItem;
  private DesktopBus desktopBus;
  private SelectionHandler<Item> startMenuListener;
  private SelectHandler shortcutListener;
  private OpenFileModelHandler openFileModelHandler;
  private Presenter presenter;
  private UpdateSettingsHandler updateSettingsHandler;

  public DesktopAppView(DesktopBus desktopBus, Presenter presenter) {
    this.desktopBus = desktopBus;
    this.presenter = presenter;
    desktopBus.addOpenFileModelHandler(getOpenFileModelHandler());
    desktopBus.addUpdateSettingsHandler(getUpdateSettingsHandler());
  }

  public Desktop getDesktop() {
    if (desktop == null) {
      desktop = new Desktop();
      desktop.addShortcut(getFileManagerShortcut());
      desktop.addStartMenuItem(getFileManagerStartMenuItem());
      desktop.setStartMenuHeading(getPresenter().getCurrentUser());
      desktop.setStartMenuIcon(DesktopImages.INSTANCE.user());
      desktop.addToolMenuItem(getSettingsToolMenuItem());
      desktop.addToolSeparator();
      desktop.addToolMenuItem(getLogoutToolMenuItem());
    }
    return desktop;
  }

  @Override
  public void go(HasWidgets hasWidgets) {
    getDesktop().go(hasWidgets);
  }

  private void activate(AsWindow asWindow) {
    Window window = asWindow.asWindow();
    desktop.activate(window);
  }

  private DesktopBus getDesktopBus() {
    return desktopBus;
  }

  private FileManagerView getFileManager() {
    if (fileManagerView == null) {
      fileManagerView = new FileManagerView(getDesktopBus(), getPresenter().getFileSystem());
    }
    return fileManagerView;
  }

  private Shortcut getFileManagerShortcut() {
    if (fileManagerShortcut == null) {
      fileManagerShortcut = new Shortcut();
      fileManagerShortcut.setText("File Manager");
      fileManagerShortcut.setIcon(DesktopImages.INSTANCE.folder_shortcut());
      fileManagerShortcut.setData(DESKLET_KEY, getFileManager());
      fileManagerShortcut.addSelectHandler(getShortcutListener());
    }
    return fileManagerShortcut;
  }

  private StartMainMenuItem getFileManagerStartMenuItem() {
    if (fileManagerStartMenuItem == null) {
      fileManagerStartMenuItem = new StartMainMenuItem("File Manager");
      fileManagerStartMenuItem.setData(DESKLET_KEY, getFileManager());
      fileManagerStartMenuItem.setIcon(DesktopImages.INSTANCE.folder());
      fileManagerStartMenuItem.addSelectionHandler(getStartMenuListener());
    }
    return fileManagerStartMenuItem;
  }

  private StartToolMenuItem getLogoutToolMenuItem() {
    if (logoutToolMenuItem == null) {
      logoutToolMenuItem = new StartToolMenuItem("Logout");
      logoutToolMenuItem.setIcon(DesktopImages.INSTANCE.door_out());
      logoutToolMenuItem.addSelectionHandler(new SelectionHandler<Item>() {
        @Override
        public void onSelection(SelectionEvent<Item> event) {
          getPresenter().logout();
        }
      });
    }
    return logoutToolMenuItem;
  }

  private OpenFileModelHandler getOpenFileModelHandler() {
    if (openFileModelHandler == null) {
      openFileModelHandler = new OpenFileModelHandler() {
        @Override
        public void onOpenFileModel(OpenFileModelEvent openFileModelEvent) {
          openFileModel(openFileModelEvent);
        }
      };
    }
    return openFileModelHandler;
  }

  private Presenter getPresenter() {
    return presenter;
  }

  private StartToolMenuItem getSettingsToolMenuItem() {
    if (settingsToolMenuItem == null) {
      settingsToolMenuItem = new StartToolMenuItem("Settings");
      settingsToolMenuItem.setIcon(DesktopImages.INSTANCE.user_edit());
      settingsToolMenuItem.addSelectionHandler(new SelectionHandler<Item>() {
        @Override
        public void onSelection(SelectionEvent<Item> event) {
          getDesktopBus().requestSettingsService(getPresenter().getSettings(getPresenter().getCurrentUser()));
        }
      });
    }
    return settingsToolMenuItem;
  }

  private SelectHandler getShortcutListener() {
    if (shortcutListener == null) {
      shortcutListener = new SelectHandler() {
        @Override
        public void onSelect(SelectEvent event) {
          activate(((Component) event.getSource()).<AsWindow> getData(DESKLET_KEY));
        }
      };
    }
    return shortcutListener;
  }

  private SelectionHandler<Item> getStartMenuListener() {
    if (startMenuListener == null) {
      startMenuListener = new SelectionHandler<Item>() {
        @Override
        public void onSelection(SelectionEvent<Item> event) {
          activate(event.getSelectedItem().<AsWindow> getData(DESKLET_KEY));
        }
      };
    }
    return startMenuListener;
  }

  private UpdateSettingsHandler getUpdateSettingsHandler() {
    if (updateSettingsHandler == null) {
      updateSettingsHandler = new UpdateSettingsHandler() {
        @Override
        public void onUpdateSettings(UpdateSettingsEvent updateSettingsEvent) {
          getDesktop().setDesktopLayoutType(updateSettingsEvent.getSettings().getDesktopLayoutType());
        }
      };
    }
    return updateSettingsHandler;
  }

  private void openFileModel(OpenFileModelEvent openFileModelEvent) {
    AsWindow asWindow = null;
    FileSystem fileSystem = openFileModelEvent.getFileSystem();
    FileModel fileModel = openFileModelEvent.getFileModel();
    switch (fileModel.getFileType()) {
      case DOCUMENT:
        asWindow = new WordProcessorView(getDesktopBus(), fileSystem, fileModel);
        break;
      case FOLDER:
        break;
      case URL:
        asWindow = new BrowserView(getDesktopBus(), fileSystem, fileModel);
        break;
      default:
        Info.display("DesktopApp", "You opened " + fileModel.getName());
        break;
    }
    if (asWindow != null) {
      activate(asWindow);
    }
  }

}
