/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp.browser;

import com.google.web.bindery.event.shared.HandlerRegistration;
import com.sencha.gxt.desktop.client.desktopapp.DesktopBus;
import com.sencha.gxt.desktop.client.desktopapp.event.RemoveFileModelEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.RemoveFileModelHandler;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateFileModelEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateFileModelHandler;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileSystem;

public class BrowserPresenter implements Presenter {

  private DesktopBus desktopBus;
  private FileSystem fileSystem;
  private FileModel fileModel;

  private View view;
  private HandlerRegistration updateFileModelRegistration;
  private UpdateFileModelHandler updateFileModelHandler;
  private HandlerRegistration removeFileModelRegistration;
  private RemoveFileModelHandler removeFileModelHandler;
  private boolean isLocalUpdate;

  public BrowserPresenter(DesktopBus desktopBus, FileSystem fileSystem, FileModel fileModel, View view) {
    this.desktopBus = desktopBus;
    this.fileSystem = fileSystem;
    this.fileModel = fileModel;
    this.view = view;
  }

  @Override
  public void bind() {
    getUpdateFileModelRegistration();
    getRemoveFileModelRegistration();
    setValue(getFileModel().getContent());
  }

  @Override
  public String getTitle() {
    return "Browser - " + getFileSystem().getPath(fileModel);
  }

  @Override
  public void onClose() {
  }

  @Override
  public void onSave() {
  }

  @Override
  public void unbind() {
    getUpdateFileModelRegistration().removeHandler();
    getRemoveFileModelRegistration().removeHandler();
  }

  private RemoveFileModelHandler getRemoveFileModelHandler() {
    if (removeFileModelHandler == null) {
      removeFileModelHandler = new RemoveFileModelHandler() {
        @Override
        public void onRemoveFileModel(RemoveFileModelEvent removeFileModelEvent) {
          onRemoveFileModelEvent(removeFileModelEvent);
        }
      };
    }
    return removeFileModelHandler;
  }

  private HandlerRegistration getRemoveFileModelRegistration() {
    if (removeFileModelRegistration == null) {
      removeFileModelRegistration = getDesktopBus().addRemoveFileModelHandler(getRemoveFileModelHandler());
    }
    return removeFileModelRegistration;
  }

  private DesktopBus getDesktopBus() {
    return desktopBus;
  }

  private FileModel getFileModel() {
    return fileModel;
  }

  private FileSystem getFileSystem() {
    return fileSystem;
  }

  private UpdateFileModelHandler getUpdateFileModelHandler() {
    if (updateFileModelHandler == null) {
      updateFileModelHandler = new UpdateFileModelHandler() {
        @Override
        public void onUpdateFileModel(UpdateFileModelEvent updateFileModelEvent) {
          onUpdateFileModelEvent(updateFileModelEvent);
        }
      };
    }
    return updateFileModelHandler;
  }

  private HandlerRegistration getUpdateFileModelRegistration() {
    if (updateFileModelRegistration == null) {
      updateFileModelRegistration = getDesktopBus().addUpdateFileModelHandler(getUpdateFileModelHandler());
    }
    return updateFileModelRegistration;
  }

  private View getView() {
    return view;
  }

  private void onRemoveFileModelEvent(RemoveFileModelEvent removeFileModelEvent) {
    FileModel fileModel = removeFileModelEvent.getFileModel();
    if (fileModel == getFileModel()) {
      getView().close();
    }
  }

  private void onUpdateFileModelEvent(UpdateFileModelEvent updateFileModelEvent) {
    if (!isLocalUpdate) {
      FileModel fileModel = updateFileModelEvent.getFileModel();
      if (fileModel == getFileModel()) {
        updateTitle();
        updateContent(fileModel);
      }
    }
  }

  private void setValue(String content) {
    getView().setValue(content);
  }

  private void updateContent(FileModel fileModel) {
  }

  private void updateTitle() {
    String newTitle = getTitle();
    getView().setTitle(newTitle);
  }

}
