/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp.filemanager;

import java.util.List;

import com.sencha.gxt.desktop.client.desktopapp.DesktopBus;
import com.sencha.gxt.desktop.client.desktopapp.event.OpenFileModelEvent;
import com.sencha.gxt.desktop.client.desktopapp.event.SelectFileModelEvent;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileSystem;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel.FileType;

public class FileManagerPresenter implements Presenter {

  private FileSystem fileSystem;
  private DesktopBus desktopBus;
  private View view;

  private boolean isNewlyCreated;

  public FileManagerPresenter(FileSystem fileSystem, DesktopBus desktopBus, View view) {
    this.fileSystem = fileSystem;
    this.desktopBus = desktopBus;
    this.view = view;
  }

  @Override
  public void bind() {
  }

  public DesktopBus getDesktopBus() {
    return desktopBus;
  }

  @Override
  public boolean isEnableCreate() {
    FileModel selectedItem = getView().getSelectedItem();
    FileType fileType = selectedItem == null ? null : selectedItem.getFileType();
    return fileType == null || fileType == FileType.FOLDER;
  }

  @Override
  public boolean isEnableDelete() {
    FileModel selectedItem = getView().getSelectedItem();
    return selectedItem != null;
  }

  @Override
  public boolean isEnableEditName() {
    FileModel selectedItem = getView().getSelectedItem();
    return selectedItem != null;
  }

  @Override
  public boolean isEnableOpen() {
    FileModel selectedItem = getView().getSelectedItem();
    FileType fileType = selectedItem == null ? null : selectedItem.getFileType();
    return fileType == FileType.DOCUMENT || fileType == FileType.PROGRAM || fileType == FileType.SPREADSHEET
        || fileType == FileType.URL;
  }

  @Override
  public void onCollapse() {
    getView().collapse();
  }

  @Override
  public void onCreate(FileType fileType) {
    FileModel parentFileModel = getView().getSelectedItem();
    String name = fileSystem.getNextUntitledFileName(parentFileModel, fileType);
    FileModel childFileModel = fileSystem.createFileModel(parentFileModel, name, fileType);
    getView().selectFileModel(childFileModel);
    isNewlyCreated = true;
    getView().editName(childFileModel);
  }

  @Override
  public void onDelete() {
    List<FileModel> fileModels = getView().getSelectedItems();
    for (FileModel fileModel : fileModels) {
      fileSystem.remove(fileModel);
    }
  }

  @Override
  public void onEditFileNameComplete(boolean isSaved) {
    if (isNewlyCreated) {
      isNewlyCreated = false;
      if (isSaved) {
        onOpen();
      } else {
        onDelete();
      }
    }
  }

  @Override
  public void onEditName() {
    FileModel selectedItem = getView().getSelectedItem();
    if (selectedItem != null) {
      getView().editName(selectedItem);
    }
  }

  @Override
  public void onExpand() {
    getView().expand();
  }

  @Override
  public void onOpen() {
    List<FileModel> fileModels = getView().getSelectedItems();
    for (FileModel fileModel : fileModels) {
      getDesktopBus().fireOpenFileModelEvent(new OpenFileModelEvent(fileSystem, fileModel));
    }
  }

  @Override
  public void onSelect(FileModel fileModel) {
    getDesktopBus().fireSelectFileModelEvent(new SelectFileModelEvent(fileModel));
  }

  @Override
  public void unbind() {
  }

  private View getView() {
    return view;
  }
}
