/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp.filemanager;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.google.gwt.cell.client.DateCell;
import com.google.gwt.dom.client.Element;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.i18n.client.DateTimeFormat.PredefinedFormat;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.data.shared.IconProvider;
import com.sencha.gxt.desktop.client.desktopapp.DesktopBus;
import com.sencha.gxt.desktop.client.desktopapp.filemanager.images.Images;
import com.sencha.gxt.desktop.client.desktopapp.has.AsWindow;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel.FileType;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModelProperties;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileSystem;
import com.sencha.gxt.dnd.core.client.TreeGridDragSource;
import com.sencha.gxt.dnd.core.client.TreeGridDropTarget;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.CancelEditEvent;
import com.sencha.gxt.widget.core.client.event.CancelEditEvent.CancelEditHandler;
import com.sencha.gxt.widget.core.client.event.CompleteEditEvent;
import com.sencha.gxt.widget.core.client.event.CompleteEditEvent.CompleteEditHandler;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.RowDoubleClickEvent;
import com.sencha.gxt.widget.core.client.event.RowDoubleClickEvent.RowDoubleClickHandler;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent.ShowHandler;
import com.sencha.gxt.widget.core.client.form.TextField;
import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
import com.sencha.gxt.widget.core.client.grid.ColumnModel;
import com.sencha.gxt.widget.core.client.grid.Grid.GridCell;
import com.sencha.gxt.widget.core.client.grid.editing.AbstractGridEditing;
import com.sencha.gxt.widget.core.client.grid.editing.GridEditing;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuBar;
import com.sencha.gxt.widget.core.client.menu.MenuBarItem;
import com.sencha.gxt.widget.core.client.selection.SelectionChangedEvent;
import com.sencha.gxt.widget.core.client.selection.SelectionChangedEvent.SelectionChangedHandler;
import com.sencha.gxt.widget.core.client.treegrid.TreeGrid;

public class FileManagerView implements AsWindow, View, IsWidget, HideHandler, ShowHandler {

  private DesktopBus desktopBus;
  private FileSystem fileSystem;
  private Presenter presenter;

  private Window window;
  private VerticalLayoutContainer verticalLayoutContainer;
  private TreeGrid<FileModel> treeGrid;
  private ColumnConfig<FileModel, FileType> typeConfig;
  private ColumnConfig<FileModel, String> nameConfig;
  private ColumnConfig<FileModel, Date> dateConfig;
  private ColumnConfig<FileModel, Long> sizeConfig;
  private ColumnModel<FileModel> columnModel;
  private FileManagerIconProvider fileManagerIconProvider;
  private FileManagerSizeCell fileManagerSizeCell;
  private FileManagerGridInlineEditing gridEditing;
  private MenuBar menuBar;
  private MenuBarItem fileMenuBarItem;
  private Menu fileMenu;
  private SelectionChangedHandler<FileModel> selectionChangedHandler;
  private TreeGridDragSource<FileModel> treeGridDragSource;
  private TreeGridDropTarget<FileModel> treeGridDropTarget;
  private Menu treeGridContextMenu;
  private RowDoubleClickHandler rowDoubleClickHandler;
  private CompleteEditHandler<FileModel> completeEditHandler;
  private CancelEditHandler<FileModel> cancelEditHandler;

  public FileManagerView(DesktopBus desktopBus, FileSystem fileSystem) {
    this.desktopBus = desktopBus;
    this.fileSystem = fileSystem;
  }

  @Override
  public Widget asWidget() {
    return getVerticalLayoutContainer();
  }

  @Override
  public Window asWindow() {
    if (window == null) {
      window = new Window();
      window.setHeadingText("File Manager");
      window.getHeader().setIcon(Images.getImageResources().folder());
      window.setMinimizable(true);
      window.setMaximizable(true);
      window.setPixelSize(500, 400);
      window.setOnEsc(false);
      window.addHideHandler(this);
      window.addShowHandler(this);
      window.add(getVerticalLayoutContainer().asWidget());
    }
    return window;
  }

  @Override
  public void collapse() {
    FileModel fileModel = getTreeGrid().getSelectionModel().getSelectedItem();
    if (fileModel == null) {
      getTreeGrid().collapseAll();
    } else {
      getTreeGrid().setExpanded(fileModel, false, true);
    }
  }

  @Override
  public void editName(FileModel fileModel) {
    Element row = getTreeGrid().getView().getRow(fileModel);
    int rowIndex = getTreeGrid().getView().findRowIndex(row);
    getGridEditing().startEditing(new GridCell(rowIndex, 0));
  }

  @Override
  public void expand() {
    FileModel fileModel = getTreeGrid().getSelectionModel().getSelectedItem();
    if (fileModel == null) {
      getTreeGrid().expandAll();
    } else {
      getTreeGrid().setExpanded(fileModel, true, true);
    }
  }

  @Override
  public FileModel getSelectedItem() {
    return getTreeGrid().getSelectionModel().getSelectedItem();
  }

  @Override
  public List<FileModel> getSelectedItems() {
    return getTreeGrid().getSelectionModel().getSelectedItems();
  }

  @Override
  public void onHide(HideEvent event) {
    getPresenter().unbind();
  }

  @Override
  public void onShow(ShowEvent event) {
    getPresenter().bind();
  }

  @Override
  public void selectFileModel(FileModel fileModel) {
    getTreeGrid().setExpanded(fileModel, true);
    getTreeGrid().getSelectionModel().select(fileModel, false);
  }

  FileSystem getFileSystem() {
    return fileSystem;
  }

  private CancelEditHandler<FileModel> getCancelEditHandler() {
    if (cancelEditHandler == null) {
      cancelEditHandler = new CancelEditHandler<FileModel>() {
        @Override
        public void onCancelEdit(CancelEditEvent<FileModel> event) {
          if (gridEditing.isEnter()) {
            // pressing enter immediately on startEditing is currently
            // reported as a cancel, not a complete
            getPresenter().onEditFileNameComplete(true);
          } else {
            getPresenter().onEditFileNameComplete(false);
          }
        }
      };
    }
    return cancelEditHandler;
  }

  private ColumnModel<FileModel> getColumnModel() {
    if (columnModel == null) {
      List<ColumnConfig<FileModel, ?>> columnConfigs = new ArrayList<ColumnConfig<FileModel, ?>>();
      columnConfigs.add(getNameConfig());
      columnConfigs.add(getDateConfig());
      columnConfigs.add(getTypeConfig());
      columnConfigs.add(getSizeConfig());
      columnModel = new ColumnModel<FileModel>(columnConfigs);
    }
    return columnModel;
  }

  private CompleteEditHandler<FileModel> getCompleteEditHandler() {
    if (completeEditHandler == null) {
      completeEditHandler = new CompleteEditHandler<FileModel>() {
        @Override
        public void onCompleteEdit(CompleteEditEvent<FileModel> event) {
          getPresenter().onEditFileNameComplete(true);
        }
      };
    }
    return completeEditHandler;
  }

  private ColumnConfig<FileModel, Date> getDateConfig() {
    if (dateConfig == null) {
      dateConfig = new ColumnConfig<FileModel, Date>(getFileModelProperties().lastModified(), 100, "Date");
      dateConfig.setCell(new DateCell(DateTimeFormat.getFormat(PredefinedFormat.DATE_TIME_MEDIUM)));
    }
    return dateConfig;
  }

  private DesktopBus getDesktopBus() {
    return desktopBus;
  }

  private IconProvider<FileModel> getFileManagerIconProvider() {
    if (fileManagerIconProvider == null) {
      fileManagerIconProvider = new FileManagerIconProvider();
    }
    return fileManagerIconProvider;
  }

  private FileManagerSizeCell getFileManagerSizeCell() {
    if (fileManagerSizeCell == null) {
      fileManagerSizeCell = new FileManagerSizeCell(getFileSystem());
    }
    return fileManagerSizeCell;
  }

  private Menu getFileMenu() {
    if (fileMenu == null) {
      fileMenu = new FileMenu(getPresenter()).asMenu();
    }
    return fileMenu;
  }

  private MenuBarItem getFileMenuBarItem() {
    if (fileMenuBarItem == null) {
      fileMenuBarItem = new MenuBarItem("File");
      fileMenuBarItem.setMenu(getFileMenu());
    }
    return fileMenuBarItem;
  }

  private FileModelProperties getFileModelProperties() {
    return getFileSystem().getFileModelProperties();
  }

  @SuppressWarnings("rawtypes")
  private GridEditing<FileModel> getGridEditing() {
    if (gridEditing == null) {
      gridEditing = new FileManagerGridInlineEditing(getTreeGrid());
      ((AbstractGridEditing) gridEditing).setClicksToEdit(null);
      gridEditing.addEditor(getNameConfig(), new TextField());
      gridEditing.addCompleteEditHandler(getCompleteEditHandler());
      gridEditing.addCancelEditHandler(getCancelEditHandler());
    }
    return gridEditing;
  }

  private MenuBar getMenuBar() {
    if (menuBar == null) {
      menuBar = new MenuBar();
      menuBar.add(getFileMenuBarItem());
    }
    return menuBar;
  }

  private ColumnConfig<FileModel, String> getNameConfig() {
    if (nameConfig == null) {
      nameConfig = new ColumnConfig<FileModel, String>(getFileModelProperties().name(), 200, "Name");
    }
    return nameConfig;
  }

  private Presenter getPresenter() {
    if (presenter == null) {
      presenter = new FileManagerPresenter(getFileSystem(), getDesktopBus(), this);
    }
    return presenter;
  }

  private RowDoubleClickHandler getRowDoubleClickHandler() {
    if (rowDoubleClickHandler == null) {
      rowDoubleClickHandler = new RowDoubleClickHandler() {
        @Override
        public void onRowDoubleClick(RowDoubleClickEvent event) {
          getPresenter().onOpen();
        }
      };
    }
    return rowDoubleClickHandler;
  }

  private SelectionChangedHandler<FileModel> getSelectionChangedHandler() {
    if (selectionChangedHandler == null) {
      selectionChangedHandler = new SelectionChangedHandler<FileModel>() {
        @Override
        public void onSelectionChanged(SelectionChangedEvent<FileModel> event) {
          FileModel fileModel = treeGrid.getSelectionModel().getSelectedItem();
          if (fileModel != null) {
            getPresenter().onSelect(fileModel);
          }
        }
      };
    }
    return selectionChangedHandler;
  }

  private ColumnConfig<FileModel, Long> getSizeConfig() {
    if (sizeConfig == null) {
      sizeConfig = new ColumnConfig<FileModel, Long>(getFileModelProperties().size(), 100, "Size");
      sizeConfig.setCell(getFileManagerSizeCell());
    }
    return sizeConfig;
  }

  private TreeGrid<FileModel> getTreeGrid() {
    if (treeGrid == null) {
      treeGrid = new FileManagerTreeGrid(getFileSystem(), getColumnModel(), getNameConfig());
      treeGrid.getView().setEmptyText("Use file or context menu to create files and folders.");
      treeGrid.setBorders(true);
      treeGrid.setTrackMouseOver(false);
      treeGrid.getView().setForceFit(true);
      treeGrid.getView().setAutoFill(true);
      treeGrid.setIconProvider(getFileManagerIconProvider());
      treeGrid.setContextMenu(getTreeGridContextMenu());
      treeGrid.getSelectionModel().addSelectionChangedHandler(getSelectionChangedHandler());
      treeGrid.addRowDoubleClickHandler(getRowDoubleClickHandler());
      getGridEditing();
      getTreeGridDragSource();
      getTreeGridDropTarget();
    }
    return treeGrid;
  }

  private Menu getTreeGridContextMenu() {
    if (treeGridContextMenu == null) {
      treeGridContextMenu = new FileMenu(getPresenter()).asMenu();
    }
    return treeGridContextMenu;
  }

  private TreeGridDragSource<FileModel> getTreeGridDragSource() {
    if (treeGridDragSource == null) {
      treeGridDragSource = new FileManagerDragSource(getTreeGrid());
    }
    return treeGridDragSource;
  }

  private TreeGridDropTarget<FileModel> getTreeGridDropTarget() {
    if (treeGridDropTarget == null) {
      treeGridDropTarget = new FileManagerDropTarget(getFileSystem(), getTreeGrid());
    }
    return treeGridDropTarget;
  }

  private ColumnConfig<FileModel, ?> getTypeConfig() {
    if (typeConfig == null) {
      typeConfig = new ColumnConfig<FileModel, FileType>(getFileModelProperties().fileType(), 100, "Type");
    }
    return typeConfig;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer();
      verticalLayoutContainer.add(getMenuBar(), new VerticalLayoutData(1, -1));
      verticalLayoutContainer.add(getTreeGrid(), new VerticalLayoutData(1, 1));
    }
    return verticalLayoutContainer;
  }

}
