/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.desktopapp.persistence;

import java.util.Date;
import java.util.List;

import com.google.web.bindery.autobean.shared.AutoBean;
import com.google.web.bindery.autobean.shared.AutoBeanFactory;
import com.sencha.gxt.desktop.client.desktopapp.persistence.FileModel.FileType;
import com.sencha.gxt.desktop.client.desktopapp.utility.Utility;

public class FileSystem extends PersistentTreeStore<FileModel> {
  private static final String FS_PREFIX = "f";
  private FileModelProperties fileModelProperties;

  public FileSystem(FileModelProperties fileModelProperties, BackingStore backingStore, AutoBeanFactory dataFactory,
      Class<FileModel> dataClass) {
    super(fileModelProperties.key(), backingStore, dataFactory, dataClass);
    this.fileModelProperties = fileModelProperties;
    setAutoCommit(true);
  }

  public FileModel createFileModel(FileModel parentFileModel, String name, FileType fileType) {
    AutoBean<FileModel> childFileModelAutoBean = getDataFactory().create(getDataClass());
    FileModel childFileModel = childFileModelAutoBean.as();
    childFileModel.setName(name);
    childFileModel.setFileType(fileType);
    childFileModel.setLastModified(new Date());
    childFileModel.setId(allocateId(FS_PREFIX));
    childFileModel.setSize(0l);
    if (parentFileModel == null) {
      add(childFileModel);
    } else {
      add(parentFileModel, childFileModel);
      TreeModel parentTreeModel = getWrapper(parentFileModel);
      parentFileModel.setSize((long) parentTreeModel.getChildren().size());
      update(parentFileModel);
    }
    return childFileModel;
  }

  public FileModelProperties getFileModelProperties() {
    return fileModelProperties;
  }

  public String getNextUntitledFileName(FileModel parentFileModel, FileType fileType) {
    String nextUntitledFileName;
    List<FileModel> children;
    if (parentFileModel == null) {
      children = getRootItems();
    } else {
      children = getChildren(parentFileModel);
    }
    int index = 1;
    String fileNameTemplate = getFileNameTemplate(fileType);
    do {
      nextUntitledFileName = fileNameTemplate + index;
      index++;
    } while (containsName(children, nextUntitledFileName));
    return nextUntitledFileName;
  }

  public String getPath(FileModel fileModel) {
    StringBuilder s = new StringBuilder();
    while (fileModel != null) {
      String name = fileModel.getName();
      s.insert(0, "/" + name);
      @SuppressWarnings("unchecked")
      PersistentTreeModel childTreeModel = (PersistentTreeModel) getWrapper(fileModel);
      @SuppressWarnings("unchecked")
      PersistentTreeModel parentTreeModel = (PersistentTreeModel) childTreeModel.getParent();
      fileModel = parentTreeModel == null ? null : parentTreeModel.getData();
    }
    return s.toString();
  }

  @Override
  public void update(FileModel item) {
    if (item.getFileType() != FileType.FOLDER) {
      String content = item.getContent();
      long length = content == null ? 0 : content.length();
      item.setSize(length);
    }
    super.update(item);
  }

  protected boolean containsName(List<FileModel> children, String name) {
    for (FileModel fileModel : children) {
      if (name.equals(fileModel.getName())) {
        return true;
      }
    }
    return false;
  }

  private String getFileNameTemplate(FileType fileType) {
    return fileType == FileType.URL ? "http://www.myurl.com" : Utility.capitalize(fileType.toString()) + " ";
  }

}
