/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.service.login;

import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.desktop.client.desktopapp.Presenter;
import com.sencha.gxt.desktop.client.desktopapp.event.LoginEvent;
import com.sencha.gxt.desktop.client.desktopapp.has.FireLoginEvent;
import com.sencha.gxt.desktop.client.desktopapp.images.DesktopImages;
import com.sencha.gxt.desktop.client.desktopapp.utility.Utility;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutPack;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
import com.sencha.gxt.widget.core.client.form.CheckBox;
import com.sencha.gxt.widget.core.client.form.FieldLabel;
import com.sencha.gxt.widget.core.client.form.FormPanel.LabelAlign;
import com.sencha.gxt.widget.core.client.form.PasswordField;
import com.sencha.gxt.widget.core.client.form.TextField;

// TODO: Add translatable Messages for user visible text
public class LoginWindow implements IsWidget {

  private Presenter presenter;
  private FireLoginEvent fireLoginEvent;
  private Window window;
  private VerticalLayoutContainer verticalLayoutContainer;
  private TextField userNameTextField;
  private FieldLabel userName;
  private CheckBox createNewUser;
  private FieldLabel password;
  private TextButton loginButton;
  private SelectHandler loginButtonSelectHandler;
  private VerticalLayoutData layoutData;
  private Margins margins;
  private PasswordField passwordField;

  public LoginWindow(FireLoginEvent fireLoginEvent) {
    this.fireLoginEvent = fireLoginEvent;
  }

  @Override
  public Widget asWidget() {
    return getWindow();
  }

  public void hide() {
    getWindow().hide();
  }

  public void show(Presenter presenter) {
    this.presenter = presenter;
    getWindow().setFocusWidget(getUserNameTextField());
    getWindow().show();
  }

  private CheckBox getCreateNewUser() {
    if (createNewUser == null) {
      createNewUser = new CheckBox();
      createNewUser.setBoxLabel("Create new user");
    }
    return createNewUser;
  }

  private VerticalLayoutData getLayoutData() {
    if (layoutData == null) {
      layoutData = new VerticalLayoutData(1, -1, getMargins());
    }
    return layoutData;
  }

  private TextButton getLoginButton() {
    if (loginButton == null) {
      loginButton = new TextButton("Login");
      loginButton.addSelectHandler(getLoginButtonSelectHandler());
    }
    return loginButton;
  }

  private SelectHandler getLoginButtonSelectHandler() {
    if (loginButtonSelectHandler == null) {
      loginButtonSelectHandler = new SelectHandler() {
        @Override
        public void onSelect(SelectEvent event) {
          String userName = getUserNameTextField().getCurrentValue();
          if (userName != null) {
            boolean isCreateNewUser = Boolean.TRUE.equals(getCreateNewUser().getValue());
            boolean isExistingUser = getPresenter().isExistingUser(userName);
            if (isCreateNewUser) {
              if (isExistingUser) {
                new AlertMessageBox("Login", "The specified user name already exists.").show();
              } else {
                // TODO: Modify Presenter to handle LoginEvent
                getPresenter().createNewUser(userName);
                getPresenter().login(userName);
                getWindow().hide();
                fireLoginEvent.fireLoginEvent(new LoginEvent(userName, isCreateNewUser));
              }
            } else {
              if (!isExistingUser) {
                new AlertMessageBox("Login", "The specified user name does not exist.").show();
              } else {
                getPresenter().login(userName);
                getWindow().hide();
                fireLoginEvent.fireLoginEvent(new LoginEvent(userName, isCreateNewUser));
              }
            }
          }
        }
      };
    }
    return loginButtonSelectHandler;
  }

  private Margins getMargins() {
    if (margins == null) {
      margins = new Margins(5);
    }
    return margins;
  }

  private FieldLabel getPassword() {
    if (password == null) {
      password = new FieldLabel(getPasswordField(), "Password");
    }
    return password;
  }

  private PasswordField getPasswordField() {
    if (passwordField == null) {
      passwordField = new PasswordField();
    }
    return passwordField;
  }

  private Presenter getPresenter() {
    return presenter;
  }

  private FieldLabel getUserName() {
    if (userName == null) {
      userName = new FieldLabel(getUserNameTextField(), "User Name");
    }
    return userName;
  }

  private TextField getUserNameTextField() {
    if (userNameTextField == null) {
      userNameTextField = new TextField();
      userNameTextField.setAllowBlank(false);
    }
    return userNameTextField;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer();
      verticalLayoutContainer.add(getUserName(), getLayoutData());
      verticalLayoutContainer.add(getPassword(), getLayoutData());
      verticalLayoutContainer.add(getCreateNewUser(), getLayoutData());
      Utility.alignLabels(LabelAlign.TOP, verticalLayoutContainer);
    }
    return verticalLayoutContainer;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window();
      window.setHeadingText("Desktop Login");
      window.getHeader().setIcon(DesktopImages.INSTANCE.door_in());
      window.setPixelSize(200, 200);
      window.setButtonAlign(BoxLayoutPack.END);
      window.setPlain(false); // TODO: resolve window background issues
      window.setModal(true);
      window.setBlinkModal(true);
      window.setClosable(false);
      window.setOnEsc(false);
      window.add(getVerticalLayoutContainer());
      window.addButton(getLoginButton());
    }
    return window;
  }

}
