/**
 * Ext GWT 3.0.0-beta3 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktop.client.service.settings;

import com.google.gwt.user.client.ui.HTML;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.core.client.util.ToggleGroup;
import com.sencha.gxt.desktop.client.desktopapp.DesktopLayoutType;
import com.sencha.gxt.desktop.client.desktopapp.Settings;
import com.sencha.gxt.desktop.client.desktopapp.event.UpdateSettingsEvent;
import com.sencha.gxt.desktop.client.desktopapp.has.FireUpdateSettingsEvent;
import com.sencha.gxt.desktop.client.desktopapp.images.DesktopImages;
import com.sencha.gxt.desktop.client.desktopapp.utility.Utility;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutPack;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;
import com.sencha.gxt.widget.core.client.form.FormPanel.LabelAlign;
import com.sencha.gxt.widget.core.client.form.Radio;

public class SettingsWindow {

  private FireUpdateSettingsEvent fireUpdateSettingsEvent;
  private Settings settings;
  private Window window;
  private VerticalLayoutContainer verticalLayoutContainer;
  private TextButton cancelButton;
  private SelectHandler cancelButtonSelectHandler;
  private HTML desktopLayoutType;
  private VerticalLayoutData layoutData;
  private Margins margins;
  private Radio tileRadio;
  private Radio cascadeRadio;
  private Radio centerRadio;
  private TextButton okayButton;
  private SelectHandler okayButtonSelectHandler;
  private ToggleGroup toggleGroup;

  public SettingsWindow(FireUpdateSettingsEvent fireUpdateSettingsEvent) {
    this.fireUpdateSettingsEvent = fireUpdateSettingsEvent;
  }

  public void show(Settings settings) {
    this.settings = settings;
    switch (settings.getDesktopLayoutType()) {
      case CASCADE:
        getCascadeRadio().setValue(true);
        break;
      case CENTER:
        getCenterRadio().setValue(true);
        break;
      case TILE:
        getTileRadio().setValue(true);
        break;
    }
    getWindow().show();
  }

  private TextButton getCancelButton() {
    if (cancelButton == null) {
      cancelButton = new TextButton("Cancel");
      cancelButton.addSelectHandler(getCancelButtonSelectHandler());
    }
    return cancelButton;
  }

  private SelectHandler getCancelButtonSelectHandler() {
    if (cancelButtonSelectHandler == null) {
      cancelButtonSelectHandler = new SelectHandler() {
        @Override
        public void onSelect(SelectEvent event) {
          getWindow().hide();
          fireUpdateSettingsEvent.fireUpdateSettingsEvent(new UpdateSettingsEvent(settings, false));
        }
      };
    }
    return cancelButtonSelectHandler;
  }

  private Radio getCascadeRadio() {
    if (cascadeRadio == null) {
      cascadeRadio = new Radio();
      cascadeRadio.setBoxLabel("Cascade");
      getToggleGroup().add(cascadeRadio);
    }
    return cascadeRadio;
  }

  private Radio getCenterRadio() {
    if (centerRadio == null) {
      centerRadio = new Radio();
      centerRadio.setBoxLabel("Center");
      getToggleGroup().add(centerRadio);
    }
    return centerRadio;
  }

  private HTML getDesktopLayoutType() {
    if (desktopLayoutType == null) {
      desktopLayoutType = new HTML("Desktop Layout Type");
    }
    return desktopLayoutType;
  }

  private VerticalLayoutData getLayoutData() {
    if (layoutData == null) {
      layoutData = new VerticalLayoutData(1, -1, getMargins());
    }
    return layoutData;
  }

  private Margins getMargins() {
    if (margins == null) {
      margins = new Margins(5);
    }
    return margins;
  }

  private TextButton getOkayButton() {
    if (okayButton == null) {
      okayButton = new TextButton("Okay");
      okayButton.addSelectHandler(getOkayButtonSelectHandler());
    }
    return okayButton;
  }

  private SelectHandler getOkayButtonSelectHandler() {
    if (okayButtonSelectHandler == null) {
      okayButtonSelectHandler = new SelectHandler() {
        @Override
        public void onSelect(SelectEvent event) {
          if (Boolean.TRUE.equals(getCascadeRadio().getValue())) {
            settings.setDesktopLayoutType(DesktopLayoutType.CASCADE);
          } else if (Boolean.TRUE.equals(getCenterRadio().getValue())) {
            settings.setDesktopLayoutType(DesktopLayoutType.CENTER);
          } else if (Boolean.TRUE.equals(getTileRadio().getValue())) {
            settings.setDesktopLayoutType(DesktopLayoutType.TILE);
          }
          getWindow().hide();
          fireUpdateSettingsEvent.fireUpdateSettingsEvent(new UpdateSettingsEvent(settings, true));
        }
      };
    }
    return okayButtonSelectHandler;
  }

  private Radio getTileRadio() {
    if (tileRadio == null) {
      tileRadio = new Radio();
      tileRadio.setBoxLabel("Tile");
      getToggleGroup().add(tileRadio);
    }
    return tileRadio;
  }

  private ToggleGroup getToggleGroup() {
    if (toggleGroup == null) {
      toggleGroup = new ToggleGroup();
    }
    return toggleGroup;
  }

  private VerticalLayoutContainer getVerticalLayoutContainer() {
    if (verticalLayoutContainer == null) {
      verticalLayoutContainer = new VerticalLayoutContainer();
      verticalLayoutContainer.add(getDesktopLayoutType(), getLayoutData());
      verticalLayoutContainer.add(getTileRadio(), getLayoutData());
      verticalLayoutContainer.add(getCascadeRadio(), getLayoutData());
      verticalLayoutContainer.add(getCenterRadio(), getLayoutData());
      Utility.alignLabels(LabelAlign.TOP, verticalLayoutContainer);
    }
    return verticalLayoutContainer;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window();
      window.setHeadingText("Settings");
      window.getHeader().setIcon(DesktopImages.INSTANCE.user_edit());
      window.setPixelSize(200, 200);
      window.setButtonAlign(BoxLayoutPack.END);
      window.setPlain(false); // TODO: set background
      window.setModal(true);
      window.setBlinkModal(true);
      window.setClosable(false);
      window.setOnEsc(false);
      window.add(getVerticalLayoutContainer());
      window.addButton(getCancelButton());
      window.addButton(getOkayButton());
    }
    return window;
  }

}
